#version 130
#extension GL_EXT_gpu_shader3 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform int lightNumber;
uniform int lightRecieve[7];

uniform sampler2D textureColorCurve;
uniform sampler2D textureImage[3];

uniform int fogMode;
uniform int useTexAlpha;
uniform int dissolveMode;
uniform int bFlat;

uniform sampler2D SM_Texture;
uniform mat4 SM_LightViewMatrix;
//uniform mat4 SM_LightProjMatrix; //KvȂB
uniform float SM_OrthoSize; //OrthoScaleKv
uniform mat4 SM_ViewMatrixInv;
uniform float SM_ProjMatrix10;
uniform float SM_ProjMatrix14;
uniform int  SM_Perspective; // -1 for not use Shadow Map
uniform int  SM_CubicMap;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

uniform float edgeThreshold;

in float t;
in vec3 position;
in vec4 position4;
in vec3 normal;
in vec4 color;
in vec4 colorDiffuse;
in vec4 colorSpecular;
flat in int blend;
flat in int textureImageIndex;

//##FUNC_SMOOTHCLIPPING##

//##FUNC_ENSEMBLE_DITHERING##

//##FUNC_FOG_EFFECT##

//##FUNC_SPOT_EFFECT##

//##FUNC_NORMAL##

//##FUNC_SHADOW_MAP##

void main (void)
{
    if (gl_FrontFacing == false) discard;
	
    vec4 texColor = vec4 (1, 1, 1, 1);
    
    switch(textureImageIndex){
    case 0:
	if (textureSize(textureImage[0], 0).x > 1)
	    texColor = texture2D(textureImage[0], gl_TexCoord[0].st);
	else
	    texColor = color;
	break;
    case 1:
	if (textureSize(textureImage[1], 0).x > 1)
	    texColor = texture2D(textureImage[1], gl_TexCoord[0].st);
	else
	    texColor = color;
	break;
    case 2:
	if (textureSize(textureImage[2], 0).x > 1)
	    texColor = texture2D(textureImage[2], gl_TexCoord[0].st);
	else
	    texColor = color;
	break;
    case 3:
	texColor = color;
    }

    float alpha = color[3];//gl_FrontLightProduct[0].ambient[3];
    /*
    if (useTexAlpha == 1){
	alpha *= texColor[3];
	if (alpha == 0) discard;
    }
    */

    float shininess = 20;
    vec3 fnormal = normalize(GetNormal());

    
    //##APPLY_SMOOTHCLIPPING##

    if (t == 0){
	gl_FragColor = vec4(0, 0, 0, alpha);
	if (fogMode != 0) FogEffect();
	//##APPLY_ENSEMBLE_DITHERING##
	return;
    }

    vec3  eye = - normalize(position);
    float eyedot = dot(eye, fnormal);
    if (eyedot * eyedot < edgeThreshold){
	gl_FragColor = vec4(0, 0, 0, alpha);
	if (fogMode != 0) FogEffect();
	//##APPLY_ENSEMBLE_DITHERING##
	return;
    }

    /*
    if (dissolveMode == 2)
	if (alpha <= 0.5) discard;
    */

    //##APPLY_ENSEMBLE_DITHERING##

    // --- Use SHADOW Map --
    float colorFactor = 1;
    if (SM_Perspective != -1){
	//##APPLY_SHADOW_MAP##
    }

    vec3 viewVec = -normalize(position);

    gl_FragColor = vec4(0, 0, 0, 1);

    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;

	float diffuseTotal = 0;
	float specularTotal = 0;

	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position.w;
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = GetSpotEffect(i, spotAngleCos);
	    
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    /*gl_FrontMaterial.shininess*/shininess) * lightFactor;
	    diffuseTotal += diffuse * length(gl_LightSource[i].diffuse) * lightFactor;
	    specularTotal += specular * length(gl_LightSource[i].specular);
	}
	
	float s = diffuseTotal * 0.8 + specularTotal;

	if (s < 0.2){
	    gl_FragColor += color * gl_LightSource[i].ambient;
	}
	else if (s < 0.75){
	    gl_FragColor += color +
		colorFactor * colorDiffuse * gl_LightSource[i].diffuse;
	}
	else{
	    gl_FragColor += color + 
		colorFactor * (colorDiffuse * gl_LightSource[i].diffuse +
			       colorSpecular * gl_LightSource[i].specular);
	}
    }

      
    if (alpha > 1.0){ 
	texColor.rgb *= alpha;
	//texColor[0] *= alpha;
	//texColor[1] *= alpha;
	//texColor[2] *= alpha; 
	alpha = 1.0;
    }

    gl_FragColor *= texColor;

    FogEffect();

    switch(blend){
    case 0:
	//gl_FragColor.rgb *= alpha;
	gl_FragColor.a = alpha;
	break;
    case 1:
	//gl_FragColor.rgb *= alpha;
	gl_FragColor.a = 0;
	break;
    case 2:
	//gl_FragColor.rgb *= alpha;
	gl_FragColor.a = alpha * (1 - alpha);
	break;
    }
}
